@@ -20,6 +20,7 @@
 #define __POPPLER_DOCUMENT_H__
 
 #include <glib-object.h>
+#include <gio/gio.h>
 #include "poppler.h"
 
 G_BEGIN_DECLS
@@ -77,17 +78,17 @@ typedef enum
 /**
  * PopplerFontType:
  * @POPPLER_FONT_TYPE_UNKNOWN: unknown font type
- * @POPPLER_FONT_TYPE_TYPE1:
- * @POPPLER_FONT_TYPE_TYPE1C:
- * @POPPLER_FONT_TYPE_TYPE1COT:
- * @POPPLER_FONT_TYPE_TYPE3:
- * @POPPLER_FONT_TYPE_TRUETYPE:
- * @POPPLER_FONT_TYPE_TRUETYPEOT:
- * @POPPLER_FONT_TYPE_CID_TYPE0:
- * @POPPLER_FONT_TYPE_CID_TYPE0C:
- * @POPPLER_FONT_TYPE_CID_TYPE0COT:
- * @POPPLER_FONT_TYPE_CID_TYPE2:
- * @POPPLER_FONT_TYPE_CID_TYPE2OT:
+ * @POPPLER_FONT_TYPE_TYPE1: Type 1 font type
+ * @POPPLER_FONT_TYPE_TYPE1C: Type 1 font type embedded in Compact Font Format (CFF) font program
+ * @POPPLER_FONT_TYPE_TYPE1COT: Type 1 font type embedded in OpenType font program
+ * @POPPLER_FONT_TYPE_TYPE3: A font type that is defined with PDF graphics operators
+ * @POPPLER_FONT_TYPE_TRUETYPE: TrueType font type
+ * @POPPLER_FONT_TYPE_TRUETYPEOT: TrueType font type embedded in OpenType font program
+ * @POPPLER_FONT_TYPE_CID_TYPE0: CIDFont type based on Type 1 font technology
+ * @POPPLER_FONT_TYPE_CID_TYPE0C: CIDFont type based on Type 1 font technology embedded in CFF font program
+ * @POPPLER_FONT_TYPE_CID_TYPE0COT: CIDFont type based on Type 1 font technology embedded in OpenType font program
+ * @POPPLER_FONT_TYPE_CID_TYPE2: CIDFont type based on TrueType font technology
+ * @POPPLER_FONT_TYPE_CID_TYPE2OT: CIDFont type based on TrueType font technology embedded in OpenType font program
  *
  * Font types
  */
@@ -116,7 +117,7 @@ typedef enum
  * @POPPLER_VIEWER_PREFERENCES_FIT_WINDOW: resize document's window to fit the size of the first displayed page
  * @POPPLER_VIEWER_PREFERENCES_CENTER_WINDOW: position the document's window in the center of the screen
  * @POPPLER_VIEWER_PREFERENCES_DISPLAY_DOC_TITLE: display document title in window's title bar
- * @POPPLER_VIEWER_PREFERENCES_DIRECTION_RTL:
+ * @POPPLER_VIEWER_PREFERENCES_DIRECTION_RTL: the predominant reading order for text is right to left
  *
  * Viewer preferences
  */
@@ -139,7 +140,12 @@ typedef enum /*< flags >*/
  * @POPPLER_PERMISSIONS_OK_TO_COPY: document can be copied
  * @POPPLER_PERMISSIONS_OK_TO_ADD_NOTES: annotations can added to the document
  * @POPPLER_PERMISSIONS_OK_TO_FILL_FORM: interactive form fields can be filled in
- * @POPPLER_PERMISSIONS_FULL:
+ * @POPPLER_PERMISSIONS_OK_TO_EXTRACT_CONTENTS: extract text and graphics
+ * (in support of accessibility to users with disabilities or for other purposes). Since 0.18
+ * @POPPLER_PERMISSIONS_OK_TO_ASSEMBLE: assemble the document (insert, rotate, or delete pages and create
+ * bookmarks or thumbnail images). Since 0.18
+ * @POPPLER_PERMISSIONS_OK_TO_PRINT_HIGH_RESOLUTION: document can be printer at high resolution. Since 0.18
+ * @POPPLER_PERMISSIONS_FULL: document permits all operations
  *
  * Permissions
  */
@@ -150,7 +156,10 @@ typedef enum /*< flags >*/
   POPPLER_PERMISSIONS_OK_TO_COPY = 1 << 2,
   POPPLER_PERMISSIONS_OK_TO_ADD_NOTES = 1 << 3,
   POPPLER_PERMISSIONS_OK_TO_FILL_FORM = 1 << 4,
-  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT | POPPLER_PERMISSIONS_OK_TO_MODIFY | POPPLER_PERMISSIONS_OK_TO_COPY | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES | POPPLER_PERMISSIONS_OK_TO_FILL_FORM)
+  POPPLER_PERMISSIONS_OK_TO_EXTRACT_CONTENTS = 1 << 5,
+  POPPLER_PERMISSIONS_OK_TO_ASSEMBLE = 1 << 6,
+  POPPLER_PERMISSIONS_OK_TO_PRINT_HIGH_RESOLUTION = 1 << 7,
+  POPPLER_PERMISSIONS_FULL = (POPPLER_PERMISSIONS_OK_TO_PRINT | POPPLER_PERMISSIONS_OK_TO_MODIFY | POPPLER_PERMISSIONS_OK_TO_COPY | POPPLER_PERMISSIONS_OK_TO_ADD_NOTES | POPPLER_PERMISSIONS_OK_TO_FILL_FORM | POPPLER_PERMISSIONS_OK_TO_EXTRACT_CONTENTS | POPPLER_PERMISSIONS_OK_TO_ASSEMBLE | POPPLER_PERMISSIONS_OK_TO_PRINT_HIGH_RESOLUTION)
 
 } PopplerPermissions;
 
@@ -164,6 +173,15 @@ PopplerDocument   *poppler_document_new_from_data          (char            *dat
 							    int              length,
 							    const char      *password,
 							    GError         **error);
+PopplerDocument   *poppler_document_new_from_stream        (GInputStream    *stream,
+                                                            goffset          length,
+                                                            const char      *password,
+                                                            GCancellable    *cancellable,
+                                                            GError         **error);
+PopplerDocument   *poppler_document_new_from_gfile         (GFile           *file,
+                                                            const char      *password,
+                                                            GCancellable    *cancellable,
+                                                            GError         **error);
 gboolean           poppler_document_save                   (PopplerDocument *document,
 							    const char      *uri,
 							    GError         **error);
@@ -197,6 +215,7 @@ PopplerPermissions poppler_document_get_permissions        (PopplerDocument *doc
 gchar             *poppler_document_get_metadata           (PopplerDocument *document);
 
 /* Attachments */
+guint              poppler_document_get_n_attachments      (PopplerDocument  *document);
 gboolean           poppler_document_has_attachments        (PopplerDocument  *document);
 GList             *poppler_document_get_attachments        (PopplerDocument  *document);
 
@@ -237,8 +256,10 @@ PopplerFontsIter *poppler_fonts_iter_copy          (PopplerFontsIter  *iter);
 void              poppler_fonts_iter_free          (PopplerFontsIter  *iter);
 const char       *poppler_fonts_iter_get_name      (PopplerFontsIter  *iter);
 const char       *poppler_fonts_iter_get_full_name (PopplerFontsIter  *iter);
+const char       *poppler_fonts_iter_get_substitute_name (PopplerFontsIter *iter);
 const char       *poppler_fonts_iter_get_file_name (PopplerFontsIter  *iter);
 PopplerFontType   poppler_fonts_iter_get_font_type (PopplerFontsIter  *iter);
+const char       *poppler_fonts_iter_get_encoding  (PopplerFontsIter *iter);
 gboolean	  poppler_fonts_iter_is_embedded   (PopplerFontsIter  *iter);
 gboolean	  poppler_fonts_iter_is_subset     (PopplerFontsIter  *iter);
 gboolean          poppler_fonts_iter_next          (PopplerFontsIter  *iter);
