@@ -15,6 +15,10 @@
 //
 // Copyright (C) 2008 Julien Rebetez <julien@fhtagn.net>
 // Copyright (C) 2009 David Benjamin <davidben@mit.edu>
+// Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2013 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -63,13 +67,19 @@ private:
 };
 
 //------------------------------------------------------------------------
-// DecryptStream
+// Helper classes
 //------------------------------------------------------------------------
 
+/* DecryptRC4State, DecryptAESState, DecryptAES256State are named like this for
+ * historical reasons, but they're used for encryption too.
+ * In case of decryption, the cbc field in AES and AES-256 contains the previous
+ * input block or the CBC initialization vector (IV) if the stream has just been
+ * reset). In case of encryption, it always contains the IV, whereas the
+ * previous output is kept in buf. The paddingReached field is only used in
+ * case of encryption. */
 struct DecryptRC4State {
   Guchar state[256];
   Guchar x, y;
-  int buf;
 };
 
 struct DecryptAESState {
@@ -77,6 +87,7 @@ struct DecryptAESState {
   Guchar state[16];
   Guchar cbc[16];
   Guchar buf[16];
+  GBool paddingReached; // encryption only
   int bufIdx;
 };
 
@@ -85,30 +96,32 @@ struct DecryptAES256State {
   Guchar state[16];
   Guchar cbc[16];
   Guchar buf[16];
+  GBool paddingReached; // encryption only
   int bufIdx;
 };
 
-class DecryptStream: public FilterStream {
+class BaseCryptStream : public FilterStream {
 public:
 
-  DecryptStream(Stream *strA, Guchar *fileKey,
-		CryptAlgorithm algoA, int keyLength,
-		int objNum, int objGen);
-  virtual ~DecryptStream();
-  virtual StreamKind getKind() { return strWeird; }
+  BaseCryptStream(Stream *strA, Guchar *fileKey, CryptAlgorithm algoA,
+                  int keyLength, int objNum, int objGen);
+  virtual ~BaseCryptStream();
+  virtual StreamKind getKind() { return strCrypt; }
   virtual void reset();
   virtual int getChar();
-  virtual int lookChar();
-  virtual int getPos();
+  virtual int lookChar() = 0;
+  virtual Goffset getPos();
   virtual GBool isBinary(GBool last);
   virtual Stream *getUndecodedStream() { return this; }
+  void setAutoDelete(GBool val);
 
-private:
-
+protected:
   CryptAlgorithm algo;
   int objKeyLength;
   Guchar objKey[32];
-  int charactersRead; // so that getPos() can be correct
+  Goffset charactersRead; // so that getPos() can be correct
+  int nextCharBuff;   // EOF means not read yet
+  GBool autoDelete;
 
   union {
     DecryptRC4State rc4;
@@ -116,11 +129,33 @@ private:
     DecryptAES256State aes256;
   } state;
 };
+
+//------------------------------------------------------------------------
+// EncryptStream / DecryptStream
+//------------------------------------------------------------------------
+
+class EncryptStream : public BaseCryptStream {
+public:
+
+  EncryptStream(Stream *strA, Guchar *fileKey, CryptAlgorithm algoA,
+                int keyLength, int objNum, int objGen);
+  ~EncryptStream();
+  virtual void reset();
+  virtual int lookChar();
+};
+
+class DecryptStream : public BaseCryptStream {
+public:
+
+  DecryptStream(Stream *strA, Guchar *fileKey, CryptAlgorithm algoA,
+                int keyLength, int objNum, int objGen);
+  ~DecryptStream();
+  virtual void reset();
+  virtual int lookChar();
+};
  
 //------------------------------------------------------------------------
 
-extern void rc4InitKey(Guchar *key, int keyLen, Guchar *state);
-extern Guchar rc4DecryptByte(Guchar *state, Guchar *x, Guchar *y, Guchar c);
 extern void md5(Guchar *msg, int msgLen, Guchar *digest);
 
 #endif
