@@ -1,7 +1,7 @@
 /* poppler-document.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, 2008, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2010, 2012, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2010, 2012, 2013, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2006-2010, Pino Toscano <pino@kde.org>
  * Copyright (C) 2010, 2011 Hib Eris <hib@hiberis.nl>
  * Copyright (C) 2012 Koji Otani <sho@bbr.jp>
@@ -269,6 +269,8 @@ namespace Poppler {
 	    return QString();
 
 	QScopedPointer<XRef> xref(m_doc->doc->getXRef()->copy());
+	if (!xref)
+		return QString();
 	xref->getDocInfo(&info);
 	if ( !info.isDict() )
 	    return QString();
@@ -300,6 +302,8 @@ namespace Poppler {
 	    return QStringList();
 
 	QScopedPointer<XRef> xref(m_doc->doc->getXRef()->copy());
+	if (!xref)
+		return QStringList();
 	xref->getDocInfo(&info);
 	if ( !info.isDict() )
 	    return QStringList();
@@ -323,6 +327,8 @@ namespace Poppler {
 
 	Object info;
 	QScopedPointer<XRef> xref(m_doc->doc->getXRef()->copy());
+	if (!xref)
+		return QDateTime();
 	xref->getDocInfo(&info);
 	if ( !info.isDict() ) {
 	    info.free();
