@@ -14,8 +14,16 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2006-2007 Albert Astals Cid <aacid@kde.org>
-// Copyright (C) 2008 Warren Toomey <wkt@tuhs.org>
+// Copyright (C) 2006, 2007, 2009, 2012 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008, 2009 Warren Toomey <wkt@tuhs.org>
+// Copyright (C) 2009, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2009 Kovid Goyal <kovid@kovidgoyal.net>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2011 Joshua Richardson <jric@chegg.com>
+// Copyright (C) 2011 Stephen Reichling <sreichling@chegg.com>
+// Copyright (C) 2012 Igor Slepchin <igor.redhat@gmail.com>
+// Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -41,7 +49,7 @@
 #include "UnicodeMap.h"
 
 
-#ifdef WIN32
+#ifdef _WIN32
 #  define SLASH '\\'
 #else
 #  define SLASH '/'
@@ -49,11 +57,12 @@
 
 #define xoutRound(x) ((int)(x + 0.5))
 
-#define DOCTYPE "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"
-#define DOCTYPE_FRAMES "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\"\n\"http://www.w3.org/TR/html4/frameset.dtd\">"
+#define DOCTYPE "<!DOCTYPE html>"
 
 class GfxState;
 class GooString;
+class PDFDoc;
+class OutlineItem;
 //------------------------------------------------------------------------
 // HtmlString
 //------------------------------------------------------------------------
@@ -80,6 +89,7 @@ public:
 	       double dx, double dy,
 	       Unicode u); 
   HtmlLink* getLink() { return link; }
+  const HtmlFont &getFont() const { return *fonts->Get(fontpos); }
   void endString(); // postprocessing
 
 private:
@@ -97,6 +107,7 @@ private:
   int len;			// length of text and xRight
   int size;			// size of text and xRight arrays
   UnicodeTextDirection dir;	// direction (left to right/right to left)
+  HtmlFontAccu *fonts;
   
   friend class HtmlPage;
 
@@ -147,7 +158,13 @@ public:
     links->AddLink(x);
   }
 
- void dump(FILE *f, int pageNum);
+  // add an image to the current page
+  void addImage(GooString *fname, GfxState *state);
+
+  // number of images on the current page
+  int  getNumImages() { return imgList->getLength(); }
+
+  void dump(FILE *f, int pageNum);
 
   // Clear the page.
   void clear();
@@ -168,17 +185,18 @@ private:
   void setDocName(char* fname);
   void dumpAsXML(FILE* f,int page);
   void dumpComplex(FILE* f, int page);
+  int dumpComplexHeaders(FILE * const file, FILE *& pageFile, int page);
 
   // marks the position of the fonts that belong to current page (for noframes)
   int fontsPageMarker; 
   HtmlFontAccu *fonts;
   HtmlLinks *links; 
+  GooList   *imgList;
   
   GooString *DocName;
   GooString *imgExt;
   int pageWidth;
   int pageHeight;
-  static int pgNum;
   int firstPage;                // used to begin the numeration of pages
 
   friend class HtmlOutputDev;
@@ -189,7 +207,7 @@ private:
 //------------------------------------------------------------------------
 class HtmlMetaVar {
 public:
-    HtmlMetaVar(char *_name, char *_content);
+    HtmlMetaVar(const char *_name, const char *_content);
     ~HtmlMetaVar();    
     
     GooString* toString();	
@@ -213,7 +231,7 @@ public:
   // 8-bit ISO Latin-1.  <useASCII7> should also be set for Japanese
   // (EUC-JP) text.  If <rawOrder> is true, the text is kept in content
   // stream order.
-  HtmlOutputDev(char *fileName, char *title, 
+  HtmlOutputDev(Catalog *catalogA, char *fileName, char *title, 
 	  char *author,
 	  char *keywords,
 	  char *subject,
@@ -250,17 +268,19 @@ public:
   virtual GBool checkPageSlice(Page *page, double hDPI, double vDPI,
                                int rotate, GBool useMediaBox, GBool crop,
                                int sliceX, int sliceY, int sliceW, int sliceH,
-                               GBool printing, Catalog * catalogA,
+                               GBool printing,
                                GBool (* abortCheckCbk)(void *data) = NULL,
-                               void * abortCheckCbkData = NULL)
+                               void * abortCheckCbkData = NULL,
+                               GBool (*annotDisplayDecideCbk)(Annot *annot, void *user_data) = NULL,
+                               void *annotDisplayDecideCbkData = NULL)
   {
-   catalog = catalogA;
+   docPage = page;
    return gTrue;
   }
 
 
   // Start a page.
-  virtual void startPage(int pageNum, GfxState *state);
+  virtual void startPage(int pageNum, GfxState *state, XRef *xref);
 
   // End a page.
   virtual void endPage();
@@ -279,10 +299,10 @@ public:
   virtual void drawImageMask(GfxState *state, Object *ref, 
 			     Stream *str,
 			     int width, int height, GBool invert,
-			     GBool inlineImg);
+			     GBool interpolate, GBool inlineImg);
   virtual void drawImage(GfxState *state, Object *ref, Stream *str,
-			  int width, int height, GfxImageColorMap *colorMap,
-			 int *maskColors, GBool inlineImg);
+			 int width, int height, GfxImageColorMap *colorMap,
+			 GBool interpolate, int *maskColors, GBool inlineImg);
 
   //new feature    
   virtual int DevType() {return 1234;}
@@ -290,17 +310,26 @@ public:
   int getPageWidth() { return maxPageWidth; }
   int getPageHeight() { return maxPageHeight; }
 
-  GBool dumpDocOutline(Catalog* catalog);
+  GBool dumpDocOutline(PDFDoc* doc);
 
 private:
   // convert encoding into a HTML standard, or encoding->getCString if not
-  // recognized
-  static char* mapEncodingToHtml(GooString* encoding);
-  void processLink(Link *link);
-  GooString* getLinkDest(Link *link,Catalog *catalog);
+  // recognized. Will delete encoding for you and return a new one
+  // that you have to delete
+  static GooString* mapEncodingToHtml(GooString* encoding);
+  void doProcessLink(AnnotLink *link);
+  GooString* getLinkDest(AnnotLink *link);
   void dumpMetaVars(FILE *);
   void doFrame(int firstPage);
-  GBool newOutlineLevel(FILE *output, Object *node, Catalog* catalog, int level = 1);
+  GBool newHtmlOutlineLevel(FILE *output, GooList *outlines, Catalog* catalog, int level = 1);
+  void newXmlOutlineLevel(FILE *output, GooList *outlines, Catalog* catalog);
+#ifndef DISABLE_OUTLINE
+  int getOutlinePageNum(OutlineItem *item);
+#endif
+  void drawJpegImage(GfxState *state, Stream *str);
+  void drawPngImage(GfxState *state, Stream *str, int width, int height,
+                    GfxImageColorMap *colorMap, GBool isMask = gFalse);
+  GooString *createImageFileName(const char *ext);
 
   FILE *fContentsFrame;
   FILE *page;                   // html file
@@ -315,11 +344,11 @@ private:
   int pageNum;
   int maxPageWidth;
   int maxPageHeight;
-  static int imgNum;
   GooString *Docname;
   GooString *docTitle;
   GooList *glMetaVars;
   Catalog *catalog;
+  Page *docPage;
   friend class HtmlPage;
 };
 
