@@ -16,10 +16,13 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2007-2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Till Kamppeter <till.kamppeter@gmail.com>
 // Copyright (C) 2009 Sanjoy Mahajan <sanjoy@mit.edu>
-// Copyright (C) 2009 William Bader <williambader@hotmail.com>
+// Copyright (C) 2009, 2011, 2012 William Bader <williambader@hotmail.com>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2012 Thomas Freitag <Thomas.Freitag@alfa.de>
+// Copyright (C) 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -44,6 +47,7 @@
 #include "Catalog.h"
 #include "Page.h"
 #include "PDFDoc.h"
+#include "PDFDocFactory.h"
 #include "PSOutputDev.h"
 #include "Error.h"
 
@@ -83,11 +87,13 @@ static GBool doForm = gFalse;
 #if OPI_SUPPORT
 static GBool doOPI = gFalse;
 #endif
+static int splashResolution = 0;
+static GBool psBinary = gFalse;
 static GBool noEmbedT1Fonts = gFalse;
 static GBool noEmbedTTFonts = gFalse;
 static GBool noEmbedCIDPSFonts = gFalse;
 static GBool noEmbedCIDTTFonts = gFalse;
-static GBool noSubstFonts = gFalse;
+static GBool fontPassthrough = gFalse;
 static GBool preload = gFalse;
 static char paperSize[15] = "";
 static int paperWidth = -1;
@@ -102,6 +108,9 @@ static char userPassword[33] = "\001";
 static GBool quiet = gFalse;
 static GBool printVersion = gFalse;
 static GBool printHelp = gFalse;
+#if SPLASH_CMYK
+static GBool overprint = gFalse;
+#endif
 
 static const ArgDesc argDesc[] = {
   {"-f",          argInt,      &firstPage,      0,
@@ -130,6 +139,10 @@ static const ArgDesc argDesc[] = {
   {"-opi",        argFlag,     &doOPI,          0,
    "generate OPI comments"},
 #endif
+  {"-r",          argInt,      &splashResolution, 0,
+   "resolution for rasterization, in DPI (default is 300)"},
+  {"-binary",     argFlag,     &psBinary,       0,
+   "write binary data in Level 1 PostScript"},
   {"-noembt1",    argFlag,     &noEmbedT1Fonts, 0,
    "don't embed Type 1 fonts"},
   {"-noembtt",    argFlag,     &noEmbedTTFonts, 0,
@@ -138,7 +151,7 @@ static const ArgDesc argDesc[] = {
    "don't embed CID PostScript fonts"},
   {"-noembcidtt", argFlag, &noEmbedCIDTTFonts,  0,
    "don't embed CID TrueType fonts"},
-  {"-passfonts",  argFlag,        &noSubstFonts,0,
+  {"-passfonts",  argFlag,        &fontPassthrough,0,
    "don't substitute missing fonts"},
   {"-preload",    argFlag,     &preload,        0,
    "preload images and forms"},
@@ -162,6 +175,10 @@ static const ArgDesc argDesc[] = {
    "owner password (for encrypted files)"},
   {"-upw",        argString,   userPassword,    sizeof(userPassword),
    "user password (for encrypted files)"},
+#if SPLASH_CMYK
+  {"-overprint",argFlag,   &overprint,      0,
+   "enable overprint"},
+#endif
   {"-q",          argFlag,     &quiet,          0,
    "don't print any messages or errors"},
   {"-v",          argFlag,     &printVersion,   0,
@@ -200,7 +217,10 @@ int main(int argc, char *argv[]) {
     if (!printVersion) {
       printUsage("pdftops", "<PDF-file> [<PS-file>]", argDesc);
     }
-    exit(1);
+    if (printVersion || printHelp)
+      exit(0);
+    else
+      exit(1);
   }
   if ((level1 ? 1 : 0) +
       (level1Sep ? 1 : 0) +
@@ -249,6 +269,11 @@ int main(int argc, char *argv[]) {
       goto err0;
     }
   }
+#if SPLASH_CMYK
+  if (overprint) {
+    globalParams->setOverprintPreview(gTrue);
+  }
+#endif  
   if (expand) {
     globalParams->setPSExpandSmaller(gTrue);
   }
@@ -261,6 +286,9 @@ int main(int argc, char *argv[]) {
   if (level1 || level1Sep || level2 || level2Sep || level3 || level3Sep) {
     globalParams->setPSLevel(level);
   }
+  if (splashResolution > 0) {
+    globalParams->setPSRasterResolution(splashResolution);
+  }
   if (noEmbedT1Fonts) {
     globalParams->setPSEmbedType1(!noEmbedT1Fonts);
   }
@@ -273,8 +301,8 @@ int main(int argc, char *argv[]) {
   if (noEmbedCIDTTFonts) {
     globalParams->setPSEmbedCIDTrueType(!noEmbedCIDTTFonts);
   }
-  if (noSubstFonts) {
-    globalParams->setPSSubstFonts(!noSubstFonts);
+  if (fontPassthrough) {
+    globalParams->setPSFontPassthrough(fontPassthrough);
   }
   if (preload) {
     globalParams->setPSPreload(preload);
@@ -284,6 +312,9 @@ int main(int argc, char *argv[]) {
     globalParams->setPSOPI(doOPI);
   }
 #endif
+  if (psBinary) {
+    globalParams->setPSBinary(psBinary);
+  }
   if (quiet) {
     globalParams->setErrQuiet(quiet);
   }
@@ -299,7 +330,13 @@ int main(int argc, char *argv[]) {
   } else {
     userPW = NULL;
   }
-  doc = new PDFDoc(fileName, ownerPW, userPW);
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
+  }
+
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
   if (userPW) {
     delete userPW;
   }
@@ -314,7 +351,7 @@ int main(int argc, char *argv[]) {
 #ifdef ENFORCE_PERMISSIONS
   // check for print permission
   if (!doc->okToPrint()) {
-    error(-1, "Printing this document is not allowed.");
+    error(errNotAllowed, -1, "Printing this document is not allowed.");
     exitCode = 3;
     goto err1;
   }
@@ -323,6 +360,9 @@ int main(int argc, char *argv[]) {
   // construct PostScript file name
   if (argc == 3) {
     psFileName = new GooString(argv[2]);
+  } else if (fileName->cmp("fd://0") == 0) {
+    error(errCommandLine, -1, "You have to provide an output filename when reading form stdin.");
+    goto err1;
   } else {
     p = fileName->getCString() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
@@ -341,16 +381,22 @@ int main(int argc, char *argv[]) {
   if (lastPage < 1 || lastPage > doc->getNumPages()) {
     lastPage = doc->getNumPages();
   }
+  if (lastPage < firstPage) {
+    error(errCommandLine, -1,
+          "Wrong page range given: the first page ({0:d}) can not be after the last page ({1:d}).",
+          firstPage, lastPage);
+    goto err2;
+  }
 
   // check for multi-page EPS or form
   if ((doEPS || doForm) && firstPage != lastPage) {
-    error(-1, "EPS and form files can only contain one page.");
+    error(errCommandLine, -1, "EPS and form files can only contain one page.");
     goto err2;
   }
 
   // write PostScript file
-  psOut = new PSOutputDev(psFileName->getCString(), doc->getXRef(),
-			  doc->getCatalog(), NULL, firstPage, lastPage, mode,
+  psOut = new PSOutputDev(psFileName->getCString(), doc,
+			  NULL, firstPage, lastPage, mode,
 			  paperWidth,
 			  paperHeight,
 			  duplex);
@@ -371,6 +417,7 @@ int main(int argc, char *argv[]) {
   delete psFileName;
  err1:
   delete doc;
+  delete fileName;
  err0:
   delete globalParams;
 
